/**** MSG_LICENCE DO NOT REMOVE ****/

#ifndef MESSAGES_H
#define MESSAGES_H

#include <stdio.h>
#include <stdarg.h>
#include "msg_datatypes.h"
#include "tbx_macros.h"
#include "simgrid_config.h"

#if HAVE_GCC == 1
#define DIE(...) FDIE(MSG_process_self(), __FILE__, __LINE__, __VA_ARGS__)
#define PRINT_MESSAGE(...) FPRINT_MESSAGE(MSG_process_self(), __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__)
#define PRINT_DEBUG_MESSAGE(...) FPRINT_DEBUG_MESSAGE(MSG_process_self(), __FILE__, __LINE__, __VA_ARGS__)

static __INLINE__ void FDIE(m_process_t self, const char *filename, int line,
			    const char *fmt, ...)
{
  va_list ap;

  va_start(ap, fmt);
  fprintf(stderr, "[%-*s, %*d] ", 12, filename, 3, line);
  vfprintf(stderr, fmt, ap);
  va_end(ap);

  abort();
  return;
}

static __INLINE__ void FPRINT_MESSAGE(m_process_t self, const char *filename,
				      int line, const char *functionname,
				      const char *fmt, ...)
{
#ifdef VERBOSE
  va_list ap;

  va_start(ap, fmt);
  if (self)
    fprintf(stderr, "[%.2f] P%d | [%s] ", MSG_getClock(),
	    MSG_process_get_PID(self), functionname);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
#endif
  return;
}
static __INLINE__ void FPRINT_DEBUG_MESSAGE(m_process_t self,
					    const char *filename, int line,
					    const char *fmt, ...)
{
#ifdef VERBOSE
  va_list ap;

  va_start(ap, fmt);
  if (self)
    fprintf(stderr, "DEBUG [%-*s, %*d] P%d | ",
	    12, filename, 3, line, MSG_process_get_PID(self));
  else
    fprintf(stderr, "DEBUG [%-*s, %*d] ", 12, filename, 3, line);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
  return;
#endif
}

#else /* no GNU CC */

static __INLINE__ void DIE(const char *fmt, ...)
{
  m_process_t self = MSG_process_self();
  va_list ap;

  va_start(ap, fmt);
  if (self)
    fprintf(stderr, "[%.2f] P%d | ", MSG_getClock(),
	    MSG_process_get_PID(self));
  vfprintf(stderr, fmt, ap);
  va_end(ap);

  abort();
  return;
}
static __INLINE__ void PRINT_MESSAGE(const char *fmt, ...)
{
#ifdef VERBOSE
  m_process_t self = MSG_process_self();
  va_list ap;

  va_start(ap, fmt);
  if (self)
    fprintf(stderr, "[%.2f] P%d | ", MSG_getClock(),
	    MSG_process_get_PID(self));
  vfprintf(stderr, fmt, ap);
  va_end(ap);
#endif
  return;
}
static __INLINE__ void PRINT_DEBUG_MESSAGE(const char *fmt, ...)
{
#ifdef VERBOSE
  m_process_t self = MSG_process_self();
  va_list ap;

  va_start(ap, fmt);
  if (self)
    fprintf(stderr, "DEBUG [%.2f] P%d | ", MSG_getClock(),
	    MSG_process_get_PID(self));
  vfprintf(stderr, fmt, ap);
  va_end(ap);
#endif
  return;
}

#endif /* GNU CC */

#endif
